#!perl

use threads;
use threads::shared;
use FindBin qw($Bin);
use lib $Bin;
use Net::NNTP;
use NNML::Server;
use NNML::Config qw($Config);
use NNmirror;

require "$Bin/config.pl";

my $flag : shared = 1;
my $inloop : shared = 0;
my $interval = shift @ARGV;
my $prefixed = "host|port|user|pass";
my $optexp = "group|" . $prefixed;

my $users = "$Bin/users.txt";
open(USERS, "<$users") or die "Cannot open $users: $!";
while (<USERS>) {
    next if /^$/ or /^#/;
    next if ! /\sadmin/;
    ($admin, $adpass) = split;
}
close(USERS);

print STDERR "start server\n";
my $server = threads->new(\&server);

print STDERR "feed loop\n";
my $feed = threads->new(\&feed_loop, $interval);

$_ = <> if $interval;
$flag = 0 if $inloop;

end_proc($server, $feed);

# end of main

sub feed_loop {
    my $interval = shift;;

    for $s (keys %site) {
	push(@ropt, nnopts($s, 'f')) if ${$site{$s}}{mode} =~ /r/;
	push(@wopt, nnopts($s, 't')) if ${$site{$s}}{mode} =~ /w/;
    }

    while ($flag) {
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)
	    = localtime(time - 3600);
	my $update = sprintf("%04d%02d%02d", 1900 + $year, ++$mon, $mday);
	my $uptime = sprintf("%02d%02d%02d", $hour, $min, $sec);

	$inloop = 1;

	call_nnmirror($update, $uptime, @ropt);
	print STDERR "NNML::Server::unspool\n";
	unspool;
	call_nnmirror($update, $uptime, @wopt);

	if ($interval and $flag) {
	    print STDERR "waiting $interval seconds\n";
	    for (1..$interval) {
		sleep 1;
		last if ! $flag;
	    }
	} else {
	    $flag = 0;
	}
    }
}

sub call_nnmirror {
    my $update = shift;
    my $uptime = shift;
    my $datefile = "$Bin/date.txt";
    my %date, %time;

    open(DATE, "<$datefile");
    while (<DATE>) {
	my ($fromto, $date, $time) = split;
	$date{$fromto} = $date;
	$time{$fromto} = $time;
    }
    close(DATE);
    for $opt (@_) {
	my $fromto = ${$opt}{fhost} . ":" . ${$opt}{thost};
	$date{$fromto} = "19700101" if ! $date{$fromto};
	$time{$fromto} = "090000" if ! $time{$fromto};
	${$opt}{date} = $date{$fromto};
	${$opt}{time} = $time{$fromto};
	my @opts;
	for $o (keys %{$opt}) {
	    push @opts, "$o => " . ${$opt}{$o};
	}
	print STDERR "nnmirror(" . join(", ", @opts) . ")\n";
	next if nnmirror(%{$opt});
	$date{$fromto} = $update;
	$time{$fromto} = $uptime;
    }
    open(DATE, ">$datefile");
    for (sort keys %date) {
	print DATE join(" ", ($_, $date{$_}, $time{$_})) . "\n";
    }
    close(DATE);
}

sub nnopts {
    my ($s, $ft) = @_;
    my $tf = ($ft eq 't') ? 'f' : 't';
    my %opt;

    $opt{"${tf}host"} = "localhost";
    $opt{"${tf}user"} = $admin;
    $opt{"${tf}pass"} = $adpass;

#    $opt{pathhost} = $s;

    for (keys %{$site{$s}}) {
	next if ! /$optexp/;
	$val = ${$site{$s}}{$_};
	s/($prefixed)/$ft$1/;
	$opt{$_} = $val;
    }

    return \%opt;
}

sub end_proc {
    my ($server, $feed) = @_;
    my $nntp = new Net::NNTP localhost, Port => $Config->port;

    $feed->join;

    $nntp->authinfo($admin, $adpass);
    $nntp->command('SHUT');
    $nntp->quit;
    $server->join;
    print STDERR "server finished\n"
}

# end of file
